import importlib.metadata

import mcp.server.stdio
import mcp.types as types
from mcp.server import NotificationOptions, Server
from mcp.server.models import InitializationOptions
from pydantic import AnyUrl

import starbridge.confluence

__version__ = importlib.metadata.version("starbridge")


class MCPServer:
    def __init__(self):
        self.notes: dict[str, str] = {}
        self.server = Server("starbridge")
        self._confluence_handler = starbridge.confluence.Handler()
        self._register_handlers()

    def _register_handlers(self):
        @self.server.list_resources()
        async def handle_list_resources() -> list[types.Resource]:
            resources = []
            resources += self._confluence_handler.resource_list()
            return resources

        @self.server.read_resource()
        async def handle_read_resource(uri: AnyUrl) -> str:
            if (uri.scheme, uri.host) == ("starbridge", "confluence"):
                return self._confluence_handler.resource_get(uri)

            raise ValueError(
                f"Unsupported URI scheme/host combination: {uri.scheme}:{uri.host}"
            )

        @self.server.list_prompts()
        async def handle_list_prompts() -> list[types.Prompt]:
            prompts = []
            prompts += starbridge.confluence.Handler.prompt_list()
            return prompts

        @self.server.get_prompt()
        async def handle_get_prompt(
            name: str, arguments: dict[str, str] | None
        ) -> types.GetPromptResult:
            if name.startswith("starbridge-confluence-"):
                method = getattr(
                    self._confluence_handler,
                    f"mcp_prompt_{name.replace('-', '_')}",
                )
                if arguments:
                    return method(**arguments)
                return method()
            return types.GetPromptResult(
                description=None,
                messages=[],
            )

        @self.server.list_tools()
        async def handle_list_tools() -> list[types.Tool]:
            tools = []
            tools += starbridge.confluence.Handler.tool_list()
            return tools

        @self.server.call_tool()
        async def handle_call_tool(
            name: str, arguments: dict | None
        ) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
            if name.startswith("starbridge-confluence-"):
                method = getattr(
                    self._confluence_handler,
                    f"mcp_tool_{name.replace('-', '_')}",
                )
                if arguments:
                    return method(**arguments)
                return method()

            raise ValueError(f"Unknown tool: {name}")

    async def run(self):
        async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
            await self.server.run(
                read_stream,
                write_stream,
                InitializationOptions(
                    server_name="starbridge",
                    server_version=__version__,
                    capabilities=self.server.get_capabilities(
                        notification_options=NotificationOptions(),
                        experimental_capabilities={},
                    ),
                ),
            )


async def run_server():
    server = MCPServer()
    await server.run()



"""
CLI to interact with Confluence
"""

import asyncio
import os

import typer

from .server import run_server

cli = typer.Typer(no_args_is_help=True)


@cli.command()
def serve(
    confluence_url: str = typer.Option(
        ..., envvar="CONFLUENCE_URL", help="Confluence url"
    ),
    confluence_email_address: str = typer.Option(
        ..., envvar="CONFLUENCE_EMAIL_ADDRESS", help="Confluence email address"
    ),
    confluence_api_token: str = typer.Option(
        ..., envvar="CONFLUENCE_API_TOKEN", help="Confluence API token"
    ),
):
    """Run MCP server."""
    os.environ.setdefault("CONFLUENCE_URL", confluence_url)
    os.environ.setdefault("CONFLUENCE_EMAIL_ADDRESS", confluence_email_address)
    os.environ.setdefault("CONFLUENCE_API_TOKEN", confluence_api_token)
    asyncio.run(run_server())



"""Handles Confluence operations."""

import inspect
import json
import os

import mcp.types as types
from atlassian import Confluence
from pydantic import AnyUrl


class Handler:
    """Handles Confluence operations."""

    def health(self) -> str:
        spaces = self.space_list()
        if len(spaces["results"]) > 0:
            return "UP"
        return "DOWN"

    @staticmethod
    def _parse_docstring_params(docstring: str) -> dict[str, str]:
        """Parse docstring to extract parameter descriptions."""
        if not docstring:
            return {}

        param_desc = {}
        lines = docstring.split("\n")
        in_args_section = False

        for line in lines:
            line = line.strip()

            # Check for Args: section
            if line.lower().startswith("args:"):
                in_args_section = True
                continue

            # We're in the Args section and have indented content
            if in_args_section and line:
                # New parameter definition
                if line and line.startswith("Returns:"):
                    in_args_section = False
                    continue

                parts = line.strip().split(":", 1)
                if len(parts) == 2:
                    param_name = parts[0].strip()
                    description = parts[1].strip()
                    if "(" in param_name:
                        param_name = param_name.split("(")[0].strip()
                    param_desc[param_name] = description

        return param_desc

    @staticmethod
    def tool_list():
        """Get available Confluence tools."""
        tools = []
        for method_name in dir(Handler):
            if method_name.startswith("mcp_tool_"):
                tool_name = method_name[9:].replace("_", "-")
                method = getattr(Handler, method_name)
                docstring = method.__doc__ or f"Call {tool_name}"
                sig = inspect.signature(method)

                # Get parameter descriptions from docstring
                param_desc = Handler._parse_docstring_params(docstring)
                # console.print(docstring)

                # Generate properties from signature
                properties = {}
                required_params = []

                for param in sig.parameters.values():
                    if param.name == "self":
                        continue

                    param_type = "string"  # default type
                    if param.annotation != inspect.Parameter.empty:
                        if param.annotation is str:
                            param_type = "string"
                        elif param.annotation is int:
                            param_type = "number"
                        elif param.annotation is bool:
                            param_type = "boolean"

                    if param.default == inspect.Parameter.empty:
                        required_params.append(param.name)

                    properties[param.name] = {
                        "type": param_type,
                        "description": param_desc.get(
                            param.name, f"Parameter {param.name}"
                        ),
                    }

                tools.append(
                    types.Tool(
                        name=tool_name,
                        description=docstring.split("\n")[0].strip(),
                        inputSchema={
                            "type": "object",
                            "required": required_params,
                            "properties": properties,
                        },
                    )
                )
        return tools

    def resource_list(self):
        spaces = self.space_list()
        return [
            types.Resource(
                uri=AnyUrl(f"starbridge://confluence/space/{space['key']}"),
                name=space["name"],
                description=f"Space of type '{space['type']}'",
                mimeType="application/json",
            )
            for space in spaces["results"]
        ]

    def resource_get(self, uri: AnyUrl) -> str:
        if (uri.scheme, uri.host) != ("starbridge", "confluence"):
            raise ValueError(
                f"Unsupported URI scheme/host combination: {uri.scheme}:{uri.host}"
            )
        if (uri.path or "").startswith("/space/"):
            space_key = uri.path.split("/")[-1]
            return json.dumps(self.space_info(space_key), indent=2)

    @staticmethod
    def prompt_list(self):
        return [
            types.Prompt(
                name="starbridge-space-summary",
                description="Creates a summary of spaces in Confluence",
                arguments=[
                    types.PromptArgument(
                        name="style",
                        description="Style of the summary (brief/detailed)",
                        required=False,
                    )
                ],
            )
        ]

    def mcp_prompt_starbridge_space_summary(
        self, style: str = "brief"
    ) -> types.GetPromptResult:
        detail_prompt = " Give extensive details." if style == "detailed" else ""
        return types.GetPromptResult(
            description="Summarize the current spaces",
            messages=[
                types.PromptMessage(
                    role="user",
                    content=types.TextContent(
                        type="text",
                        text=f"Here are the current spaces to summarize:{detail_prompt}\n\n"
                        + "\n".join(
                            f"- {space['key']}: {space['name']} ({space['type']})"
                            for space in self.space_list()["results"]
                        ),
                    ),
                )
            ],
        )

    def space_info(self, space_key):
        return self._api.get_space(space_key)

    def __init__(self):
        self._url = os.environ.get("CONFLUENCE_URL")
        self._email_address = os.environ.get("CONFLUENCE_EMAIL_ADDRESS")
        self._api_token = os.environ.get("CONFLUENCE_API_TOKEN")
        self._api = Confluence(
            url=self._url,
            username=self._email_address,
            password=self._api_token,
            cloud=True,
        )

    def info(self):
        return {
            "url": self._url,
            "email": self._email_address,
            "api_token": self._api_token,
        }

    def mcp_tool_starbridge_confluence_info(self):
        """Info about Confluence environment"""
        return [types.TextContent(type="text", text=json.dumps(self.info(), indent=2))]

    def space_list(self):
        return self._api.get_all_spaces()

    def mcp_tool_starbridge_confluence_space_list(self):
        """List spaces in Confluence"""
        return [
            types.TextContent(type="text", text=json.dumps(self.space_list(), indent=2))
        ]

    def page_create(self, space_key, title, body, parent_id=None):
        return self._api.create_page(
            space=space_key,
            title=title,
            body=body,
            parent_id=parent_id,
            type="page",
            representation="storage",
        )

    def mcp_tool_starbridge_confluence_page_create(
        self, space_key, title, body, parent_id=None
    ):
        """Create page in Confluence space given key of space, title and body of page and optional parent page id.

        Args:
            space_key (str): The key identifier of the Confluence space where the page will be created
            title (str): The title of the new page to be created
            body (str): The content/body of the new page
            parent_id (str, optional): The ID of the parent page if this is to be created as a child page. Defaults to None.

        Returns:
            list: A list containing a TextContent object with the JSON response of the page creation
        """
        return [
            types.TextContent(
                type="text",
                text=json.dumps(
                    self.page_create(
                        space_key=space_key, title=title, body=body, parent_id=parent_id
                    ),
                    indent=2,
                ),
            )
        ]



"""
CLI to interact with Confluence
"""

import typer
from pydantic import AnyUrl

from ..utils.console import console
from .handler import Handler

cli = typer.Typer(no_args_is_help=True)


@cli.command(name="info")
def info(
    ctx: typer.Context,
    verbose: bool = False,
):  # pylint: disable=W0613
    """Info about Confluence"""
    console.print(Handler().info())


cli_mcp = typer.Typer(no_args_is_help=True)
cli.add_typer(cli_mcp, name="mcp")


@cli_mcp.callback()
def mcp(ctx: typer.Context, verbose: bool = False):  # pylint: disable=W0613
    """MCP endpoints"""


@cli_mcp.command(name="tools")
def tool_list(
    ctx: typer.Context,
    verbose: bool = False,
):  # pylint: disable=W0613
    """List tools exposed via MCP"""
    console.print(Handler.tool_list())


@cli_mcp.command(name="resources")
def resources_list(
    ctx: typer.Context,
    verbose: bool = False,
):  # pylint: disable=W0613
    """List resources exposed via MCP"""
    console.print(Handler().resource_list())


@cli_mcp.command(name="resource")
def resource_get(
    ctx: typer.Context,
    verbose: bool = False,
    uri: str = typer.Option(..., help="Resource URI"),
):  # pylint: disable=W0613
    """Get resource exposed via MCP"""
    console.print(Handler().resource_get(uri=AnyUrl(uri)))


@cli_mcp.command(name="prompts")
def prompt_list(
    ctx: typer.Context,
    verbose: bool = False,
):  # pylint: disable=W0613
    """List prompts exposed via MCP"""
    console.print(Handler().prompt_list())


@cli_mcp.command(name="space-summary")
def prompt_space_summary(
    ctx: typer.Context,
    verbose: bool = False,
):  # pylint: disable=W0613
    """Summary of all spaces"""
    console.print(Handler().mcp_prompt_starbridge_space_summary())


cli_space = typer.Typer(no_args_is_help=True)
cli.add_typer(cli_space, name="space")


@cli_space.callback()
def space(ctx: typer.Context, verbose: bool = False):  # pylint: disable=W0613
    """Operations on Confluence spaces"""


@cli_space.command(name="list")
def space_list(
    ctx: typer.Context,
    verbose: bool = False,
):  # pylint: disable=W0613
    """Get info about all space"""
    console.print(Handler().space_list())


cli_page = typer.Typer(no_args_is_help=True)
cli.add_typer(cli_page, name="page")


@cli_page.callback()
def page(ctx: typer.Context, verbose: bool = False):  # pylint: disable=W0613
    """Operations on Confluence pages"""


@cli_page.command(name="create")
def page_list(
    ctx: typer.Context,
    verbose: bool = False,
    space_key: str = typer.Option(..., help="Space key"),
    title: str = typer.Option(..., help="Title of the page"),
    body: str = typer.Option(..., help="Body of the page"),
    page_id: str = typer.Option(None, help="Parent page id"),
):  # pylint: disable=W0613
    """Create a new page"""
    console.print(Handler().page_create(space_key, title, body, page_id))



import logging

import click
from rich.logging import RichHandler

from starbridge.utils.console import console


class CustomFilter(logging.Filter):
    def filter(self, record):
        return True


rich_handler = RichHandler(
    console=console,
    markup=True,
    rich_tracebacks=True,
    tracebacks_suppress=[click],
    show_path=True,
    show_level=True,
    enable_link_path=True,
)
rich_handler.addFilter(CustomFilter())

logging.basicConfig(
    level="INFO",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[rich_handler],
)

log = logging.getLogger("rich")



import importlib.metadata

import typer
from dotenv import load_dotenv

import starbridge.confluence
import starbridge.mcp

from .utils.console import console

load_dotenv()

__version__ = importlib.metadata.version("starbridge")

cli = typer.Typer(
    name="Starbridge CLI",
    no_args_is_help=True,
    help=f"Starbride (Version: {__version__})",
    epilog="Built with love in Berlin by Helmut Hoffer von Ankershoffen",
)


@cli.command()
def info():
    """Info about Starbridge Environment"""
    console.print({"version": __version__})


@cli.command()
def health():
    """Health of starbridge and dependencie"""
    dependencies = {"confluence": starbridge.confluence.Handler().health()}
    healthy = all(status == "UP" for status in dependencies.values())
    console.print({"healthy": healthy, "dependencies": dependencies})


@cli.command()
def tools():
    """Tools exposed by modules"""
    tools = []
    tools += starbridge.confluence.Handler.tool_list()
    console.print(tools)


cli.add_typer(
    starbridge.mcp.cli,
    name="mcp",
    help="MCP operations",
)
cli.add_typer(
    starbridge.confluence.cli, name="confluence", help="Confluence operations"
)


if __name__ == "__main__":
    cli()
